@@ -4,6 +4,7 @@
 //
 //    Copyright (C) 2002-2005 Renchi Raju <renchi at pooh.tam.uiuc.edu>
 //                            Gilles Caulier <caulier dot gilles at free.fr>
+//    Copyright (C) 2006 Tom Albers <tomalbers@kde.nl>
 //
 //    This program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -48,6 +49,12 @@
 #include <kdeversion.h>
 #include <kapplication.h>
 #include <kmessagebox.h>
+#include <kautomount.h>
+#include <kio/netaccess.h>
+
+using KIO::Job;
+using KIO::UDSEntryList;
+using KIO::UDSEntry;
 
 // Includes files for plugins support.
 
@@ -58,6 +65,7 @@
 // Local includes.
 
 #include "albummanager.h"
+#include "albumlister.h"
 #include "album.h"
 #include "themeengine.h"
 #include "cameralist.h"
@@ -72,8 +80,11 @@
 #include "imagewindow.h"
 #include "digikamapp.h"
 #include "splashscreen.h"
+#include "thumbnailsize.h"
+#include "scanlib.h"
+#include "dcrawbinary.h"
 
-DigikamApp::DigikamApp(bool detectCamera)
+DigikamApp::DigikamApp()
           : KMainWindow( 0, "Digikam" )
 {
     m_instance = this;
@@ -81,19 +92,24 @@ DigikamApp::DigikamApp(bool detectCamera)
 
     mFullScreen = false;
     mView = 0;
-	
+
     mSplash = 0;
     if(m_config->readBoolEntry("Show Splash", true) &&
        !kapp->isRestored())
     {
-        mSplash = new SplashScreen();
+        mSplash = new SplashScreen("digikam-splash.png");
     }
 
     mAlbumSettings = new AlbumSettings();
     mAlbumSettings->readSettings();
 
-    mAlbumManager = new AlbumManager();
-
+    mAlbumManager = AlbumManager::instance();
+    AlbumLister::instance();
+    
+    mCameraMediaList = new QPopupMenu;
+    connect( mCameraMediaList, SIGNAL( aboutToShow() ),
+             SLOT( slotCameraMediaMenu() ) );
+    
     mCameraList = new CameraList(this, locateLocal("appdata", "cameras.xml"));
 
     connect(mCameraList, SIGNAL(signalCameraAdded(CameraType *)),
@@ -109,11 +125,7 @@ DigikamApp::DigikamApp(bool detectCamera)
     applyMainWindowSettings(m_config);
 
     mAlbumManager->setLibraryPath(mAlbumSettings->getAlbumLibraryPath());
-
-    // Load Cameras
-    if(mSplash)
-        mSplash->message(i18n("Loading cameras"), AlignLeft, white);
-    loadCameras();
+    mAlbumManager->startScan();
 
     // Load KIPI Plugins.
     loadPlugins();
@@ -121,22 +133,20 @@ DigikamApp::DigikamApp(bool detectCamera)
     // Load Themes
     populateThemes();
 
-    setAutoSaveSettings();    
+    setAutoSaveSettings();
     
-    // Auto-detect camera if requested so
-    if (detectCamera)
-    {
-        if(mSplash)
-            mSplash->message(i18n("Auto-detect camera"), AlignLeft, white);
-        QTimer::singleShot(0, this, SLOT(slotCameraAutoDetect()));
-    }
+    mDcopIface = new DCOPIface(this, "camera");
+    connect(mDcopIface, SIGNAL(signalCameraAutoDetect()), 
+            this, SLOT(slotCameraAutoDetect()));
+    connect(mDcopIface, SIGNAL(signalDownloadImages( const QString & )),
+            this, SLOT(slotDownloadImages(const QString &)));
 }
 
 DigikamApp::~DigikamApp()
 {
     if (ImageWindow::imagewindow())
         delete ImageWindow::imagewindow();
-    
+
     if (mView)
         delete mView;
 
@@ -144,6 +154,9 @@ DigikamApp::~DigikamApp()
     delete mAlbumSettings;
 
     delete mAlbumManager;
+    delete AlbumLister::instance();
+
+    Digikam::DcrawBinary::cleanUp();
 
     m_instance = 0;
 }
@@ -179,6 +192,36 @@ const QPtrList<KAction>& DigikamApp::menuAlbumActions()
     return m_kipiAlbumActions;
 }
 
+const QPtrList<KAction> DigikamApp::menuImportActions()
+{
+    QPtrList<KAction> importMenu;
+    importMenu = m_kipiFileActionsImport;
+    importMenu.append( mAlbumImportAction );
+    importMenu.append( mAddImagesAction );
+    return importMenu;
+}
+
+void DigikamApp::autoDetect()
+{
+    // Auto-detect camera if requested so
+    if(mSplash)
+        mSplash->message(i18n("Auto-detect camera"), AlignLeft, white);
+    QTimer::singleShot(0, this, SLOT(slotCameraAutoDetect()));
+}
+
+void DigikamApp::downloadFrom(const QString &cameraGuiPath)
+{
+    if (!cameraGuiPath.isNull())
+    {
+        mCameraGuiPath = cameraGuiPath;
+        
+        if(mSplash)
+            mSplash->message(i18n("Opening Download Dialog"), AlignLeft, white);
+        
+        QTimer::singleShot(0, this, SLOT(slotDownloadImages()));
+    }
+}
+
 bool DigikamApp::queryClose()
 {
     return true;
@@ -227,15 +270,15 @@ void DigikamApp::setupActions()
                                     actionCollection(),
                                     "album_back");
     mBackwardActionMenu->setEnabled(false);
-    connect(mBackwardActionMenu->popupMenu(), 
+    connect(mBackwardActionMenu->popupMenu(),
             SIGNAL(aboutToShow()),
             this,
             SLOT(slotAboutToShowBackwardMenu()));
-    connect(mBackwardActionMenu->popupMenu(), 
+    connect(mBackwardActionMenu->popupMenu(),
             SIGNAL(activated(int)),
             mView,
             SLOT(slotAlbumHistoryBack(int)));
-       
+
     mForwardActionMenu = new  KToolBarPopupAction(i18n("Forward"),
                                     "forward",
                                     ALT+Key_Right,
@@ -244,15 +287,15 @@ void DigikamApp::setupActions()
                                     actionCollection(),
                                     "album_forward");
     mForwardActionMenu->setEnabled(false);
-    connect(mForwardActionMenu->popupMenu(), 
+    connect(mForwardActionMenu->popupMenu(),
             SIGNAL(aboutToShow()),
             this,
             SLOT(slotAboutToShowForwardMenu()));
-    connect(mForwardActionMenu->popupMenu(), 
+    connect(mForwardActionMenu->popupMenu(),
             SIGNAL(activated(int)),
             mView,
-            SLOT(slotAlbumHistoryForward(int)));    
-        
+            SLOT(slotAlbumHistoryForward(int)));
+
     mNewAction = new KAction(i18n("&New Album..."),
                                     "albumfoldernew",
                                     KStdAccel::shortcut(KStdAccel::New),
@@ -260,8 +303,8 @@ void DigikamApp::setupActions()
                                     SLOT(slot_newAlbum()),
                                     actionCollection(),
                                     "album_new");
-    mNewAction->setWhatsThis(i18n("This option create a new empty Album in the database."));
-                                         
+    mNewAction->setWhatsThis(i18n("This option creates a new empty Album in the database."));
+
     mAlbumSortAction = new KSelectAction(i18n("&Sort Albums"),
                                     0,
                                     0,
@@ -293,7 +336,7 @@ void DigikamApp::setupActions()
                                     SLOT(slot_albumAddImages()),
                                     actionCollection(),
                                     "album_addImages");
-    mAddImagesAction->setWhatsThis(i18n("This option adding new images in the current Album."));
+    mAddImagesAction->setWhatsThis(i18n("This option adds new images to the current Album."));
 
     mAlbumImportAction = new KAction( i18n("Import Folders..."),
                                     "fileopen",
@@ -313,7 +356,7 @@ void DigikamApp::setupActions()
                                     "album_propsEdit");
     mPropsEditAction->setWhatsThis(i18n("This option allows you to set the Album Properties information "
                                         "about the Collection."));
-    
+
     mOpenInKonquiAction = new KAction( i18n("Open in Konqueror"),
                                     "konqueror",
                                     0,
@@ -345,9 +388,9 @@ void DigikamApp::setupActions()
                                     SLOT(slot_imageView()),
                                     actionCollection(),
                                     "image_view");
-    mImageViewAction->setWhatsThis(i18n("This option allows you to open the Image Editor with the current selected "
+    mImageViewAction->setWhatsThis(i18n("This option allows you to open the Image Editor with the currently selected "
                                         "image."));
-                                        
+
     mImageCommentsAction = new KAction(i18n("Edit Image Comments && Tags..."),
                                    "imagecomment",
                                     Key_F3,
@@ -355,9 +398,9 @@ void DigikamApp::setupActions()
                                     SLOT(slot_imageCommentsEdit()),
                                     actionCollection(),
                                     "image_comments");
-    mImageCommentsAction->setWhatsThis(i18n("This option allows you to edit the comments and tags of the current "
+    mImageCommentsAction->setWhatsThis(i18n("This option allows you to edit the comments and tags of the currently "
                                             "selected image."));
-                                        
+
     mImageRenameAction = new KAction(i18n("Rename..."),
                                     "pencil",
                                     Key_F2,
@@ -365,9 +408,9 @@ void DigikamApp::setupActions()
                                     SLOT(slot_imageRename()),
                                     actionCollection(),
                                     "image_rename");
-    mImageRenameAction->setWhatsThis(i18n("This option allows you to rename the filename of the current selected "
-                                          "image."));                                    
-                                    
+    mImageRenameAction->setWhatsThis(i18n("This option allows you to rename the filename of the currently selected "
+                                          "image."));
+
     mImageDeleteAction = new KAction(i18n("Delete"),
                                     "editdelete",
                                     SHIFT+Key_Delete,
@@ -376,7 +419,7 @@ void DigikamApp::setupActions()
                                     actionCollection(),
                                     "image_delete");
 
-    mImagePropsAction = new KAction(i18n("Properties..."),
+    mImagePropsAction = new KAction(i18n("Properties"),
                                     "exifinfo",
                                     ALT+Key_Return,
                                     mView,
@@ -384,7 +427,7 @@ void DigikamApp::setupActions()
                                     actionCollection(),
                                     "image_properties");
     mImagePropsAction->setWhatsThis(i18n("This option allows you to display the file properties, the meta-data "
-                                         "and the histogram of the current selected image."));
+                                         "and the histogram of the currently selected image."));
 
     mImageSortAction = new KSelectAction(i18n("&Sort Images"),
                                     0,
@@ -401,6 +444,7 @@ void DigikamApp::setupActions()
     sortImagesActionList.append(i18n("By Path"));
     sortImagesActionList.append(i18n("By Date"));
     sortImagesActionList.append(i18n("By File Size"));
+    sortImagesActionList.append(i18n("By Rating"));
     mImageSortAction->setItems(sortImagesActionList);
 
     // -----------------------------------------------------------------
@@ -412,17 +456,26 @@ void DigikamApp::setupActions()
     mImageExifOrientationActionMenu = new KActionMenu(i18n("Correct Exif Orientation Tag"),
                                                       actionCollection(),
                                                       "image_set_exif_orientation");
+    mImageExifOrientationActionMenu->setDelayed(false);
 
-    mImageSetExifOrientation1Action = new KAction(i18n("Normal"),0,actionCollection());
-    mImageSetExifOrientation2Action = new KAction(i18n("Flipped Horizontally"),0,actionCollection());
-    mImageSetExifOrientation3Action = new KAction(i18n("Rotated 180 Degrees"),0,actionCollection());
-    mImageSetExifOrientation4Action = new KAction(i18n("Flipped Vertically"),0,actionCollection());
-    mImageSetExifOrientation5Action = new KAction(i18n("Rotated 90 Degrees / Horiz. Flipped"),
-                                                       0, actionCollection());
-    mImageSetExifOrientation6Action = new KAction(i18n("Rotated 90 Degrees"),0,actionCollection());
-    mImageSetExifOrientation7Action = new KAction(i18n("Rotated 90 Degrees / Vert. Flipped"),
-                                                       0, actionCollection());
-    mImageSetExifOrientation8Action = new KAction(i18n("Rotated 270 Degrees"),0,actionCollection());
+    mImageSetExifOrientation1Action =
+        new KAction(i18n("Normal"),0,mImageExifOrientationActionMenu);
+    mImageSetExifOrientation2Action =
+        new KAction(i18n("Flipped Horizontally"),0,mImageExifOrientationActionMenu);
+    mImageSetExifOrientation3Action =
+        new KAction(i18n("Rotated 180 Degrees"),0,mImageExifOrientationActionMenu);
+    mImageSetExifOrientation4Action =
+        new KAction(i18n("Flipped Vertically"),0,mImageExifOrientationActionMenu);
+    mImageSetExifOrientation5Action =
+        new KAction(i18n("Rotated 90 Degrees / Horiz. Flipped"),
+                    0, mImageExifOrientationActionMenu);
+    mImageSetExifOrientation6Action =
+        new KAction(i18n("Rotated 90 Degrees"),0,mImageExifOrientationActionMenu);
+    mImageSetExifOrientation7Action =
+        new KAction(i18n("Rotated 90 Degrees / Vert. Flipped"),
+                    0, mImageExifOrientationActionMenu);
+    mImageSetExifOrientation8Action =
+        new KAction(i18n("Rotated 270 Degrees"),0,mImageExifOrientationActionMenu);
 
     mImageExifOrientationActionMenu->insert(mImageSetExifOrientation1Action);
     mImageExifOrientationActionMenu->insert(mImageSetExifOrientation2Action);
@@ -492,7 +545,7 @@ void DigikamApp::setupActions()
                                    SLOT(slot_thumbSizePlus()),
                                    actionCollection(),
                                    "album_thumbSizeIncrease");
-    mThumbSizePlusAction->setWhatsThis(i18n("This option allows you to increase the Album thumbnails size."));    
+    mThumbSizePlusAction->setWhatsThis(i18n("This option allows you to increase the Album thumbnails size."));
 
     mThumbSizeMinusAction = new KAction(i18n("Decrease Thumbnail Size"),
                                    "viewmag-",
@@ -501,7 +554,7 @@ void DigikamApp::setupActions()
                                    SLOT(slot_thumbSizeMinus()),
                                    actionCollection(),
                                    "album_thumbSizeDecrease");
-    mThumbSizeMinusAction->setWhatsThis(i18n("This option allows you to decrease the Album thumbnails size."));    
+    mThumbSizeMinusAction->setWhatsThis(i18n("This option allows you to decrease the Album thumbnails size."));
 
     mFullScreenAction = new KAction(i18n("Toggle Full Screen"),
                                    "window_fullscreen",
@@ -510,8 +563,8 @@ void DigikamApp::setupActions()
                                    SLOT(slotToggleFullScreen()),
                                    actionCollection(),
                                    "full_screen");
-    mFullScreenAction->setWhatsThis(i18n("This option allows you to toggle the main windows in full screen mode."));    
-    
+    mFullScreenAction->setWhatsThis(i18n("This option allows you to toggle the main windows in full screen mode."));
+
     mQuitAction = KStdAction::quit(this,
                                    SLOT(slot_exit()),
                                    actionCollection(),
@@ -538,9 +591,53 @@ void DigikamApp::setupActions()
                                    actionCollection(),
                                    "gamma_adjustment");
 
+    // -- Rating actions ---------------------------------------------------------------
+
+    m_0Star = new KAction(i18n("No Star"), CTRL+Key_0,
+                          mView, SLOT(slotAssignRatingNoStar()),
+                          actionCollection(), "imageview_ratenostar");
+    m_1Star = new KAction(i18n("One Star"), CTRL+Key_1,
+                          mView, SLOT(slotAssignRatingOneStar()),
+                          actionCollection(), "imageview_rateonestar");
+    m_2Star = new KAction(i18n("Two Star"), CTRL+Key_2, 
+                          mView, SLOT(slotAssignRatingTwoStar()),
+                          actionCollection(), "imageview_ratetwostar");
+    m_3Star = new KAction(i18n("Three Star"), CTRL+Key_3, 
+                          mView, SLOT(slotAssignRatingThreeStar()),
+                          actionCollection(), "imageview_ratethreestar");
+    m_4Star = new KAction(i18n("Four Star"), CTRL+Key_4, 
+                          mView, SLOT(slotAssignRatingFourStar()),
+                          actionCollection(), "imageview_ratefourstar");
+    m_5Star = new KAction(i18n("Five Star"), CTRL+Key_5, 
+                          mView, SLOT(slotAssignRatingFiveStar()),
+                          actionCollection(), "imageview_ratefivestar");
+                          
+    // -----------------------------------------------------------
+
+    KAction* findAction = KStdAction::find(mView, SLOT(slotNewQuickSearch()),
+                                           actionCollection(), "search_quick");
+    findAction->setText(i18n("Quick Search..."));
+
+    findAction = KStdAction::find(mView, SLOT(slotNewAdvancedSearch()),
+                                  actionCollection(), "search_advanced");
+    findAction->setText(i18n("Advanced Search..."));
+    findAction->setShortcut("Ctrl+Alt+F");
+
+    new KAction(i18n("Scan for New Images"), "reload_page", 0,
+                this, SLOT(slotDatabaseRescan()), actionCollection(), 
+                "database_rescan");
+
+    // -----------------------------------------------------------
+
     // Provides a menu entry that allows showing/hiding the toolbar(s)
     setStandardToolBarMenuEnabled(true);
 
+    // Load Cameras -- do this before the createGUI so that the cameras
+    // are plugged into the toolbar at startup
+    if (mSplash)
+        mSplash->message(i18n("Loading cameras"), AlignLeft, white);
+    loadCameras();
+
     createGUI(QString::fromLatin1( "digikamui.rc" ), false);
 
     // Initialize Actions ---------------------------------------
@@ -560,6 +657,23 @@ void DigikamApp::setupActions()
 
     mAlbumSortAction->setCurrentItem((int)mAlbumSettings->getAlbumSortOrder());
     mImageSortAction->setCurrentItem((int)mAlbumSettings->getImageSortOrder());
+
+    int size = mAlbumSettings->getDefaultIconSize();
+    if (size == ThumbnailSize::Huge)
+    {
+        enableThumbSizePlusAction(false);
+        enableThumbSizeMinusAction(true);
+    }
+    else if (size == ThumbnailSize::Small)
+    {
+        enableThumbSizePlusAction(true);
+        enableThumbSizeMinusAction(false);
+    }
+    else
+    {
+        enableThumbSizePlusAction(true);
+        enableThumbSizeMinusAction(true);
+    }
 }
 
 void DigikamApp::enableThumbSizePlusAction(bool val)
@@ -590,11 +704,11 @@ void DigikamApp::slotAboutToShowBackwardMenu()
     if(!titles.isEmpty())
     {
         int id = 1;
-        QStringList::Iterator iter = titles.begin();        
+        QStringList::Iterator iter = titles.begin();
         for(; iter != titles.end(); ++iter,++id)
         {
             mBackwardActionMenu->popupMenu()->insertItem(*iter, id);
-        }        
+        }
     }
 }
 
@@ -610,41 +724,151 @@ void DigikamApp::slotAboutToShowForwardMenu()
         for(; iter != titles.end(); ++iter,++id)
         {
             mForwardActionMenu->popupMenu()->insertItem(*iter, id);
-        }        
+        }
     }
 }
 
 void DigikamApp::slot_albumSelected(bool val)
 {
     Album *album = mAlbumManager->currentAlbum();
-    if(album && !album->isRoot())
-    {   
-        mDeleteAction->setEnabled(val);
-        mAddImagesAction->setEnabled(val);
-        mPropsEditAction->setEnabled(val);
-        mOpenInKonquiAction->setEnabled(val);
-    }
-    else
+    
+    if(album && !val)
     {
+        // No PAlbum is selected
         mDeleteAction->setEnabled(false);
         mAddImagesAction->setEnabled(false);
         mPropsEditAction->setEnabled(false);
         mOpenInKonquiAction->setEnabled(false);
+        mNewAction->setEnabled(false);
+        mAlbumImportAction->setEnabled(false);
+
+    }
+    else if(!album && !val)
+    {
+        // Groupitem selected (Collection/date)
+        mDeleteAction->setEnabled(false);
+        mAddImagesAction->setEnabled(false);
+        mPropsEditAction->setEnabled(false);
+        mOpenInKonquiAction->setEnabled(false);
+        mNewAction->setEnabled(false);
+        mAlbumImportAction->setEnabled(false);
+        
+        KAction *action;
+        for (action = m_kipiFileActionsImport.first(); action;
+             action = m_kipiFileActionsImport.next())
+        {
+            action->setEnabled(false);
+        }
+
+        for (action = m_kipiFileActionsExport.first(); action;
+             action = m_kipiFileActionsExport.next())
+        {
+            action->setEnabled(false);
+        }
+    }
+    else if(album && !album->isRoot() && album->type() == Album::PHYSICAL)
+    {
+        // Normal Album selected
+        mDeleteAction->setEnabled(true);
+        mAddImagesAction->setEnabled(true);
+        mPropsEditAction->setEnabled(true);
+        mOpenInKonquiAction->setEnabled(true);
+        mNewAction->setEnabled(true);
+        mAlbumImportAction->setEnabled(true);
+        
+        KAction *action;
+        for (action = m_kipiFileActionsImport.first(); action; 
+             action = m_kipiFileActionsImport.next())
+        {
+            action->setEnabled(true);
+        }
+
+        for (action = m_kipiFileActionsExport.first(); action; 
+             action = m_kipiFileActionsExport.next())
+        {
+            action->setEnabled(true);    
+        }        
+    }
+    else if(album && album->isRoot() && album->type() == Album::PHYSICAL)
+    {
+        // Root Album selected
+        mDeleteAction->setEnabled(false);
+        mAddImagesAction->setEnabled(false);
+        mPropsEditAction->setEnabled(false);
+       
+
+        if(album->type() == Album::PHYSICAL)
+        {
+            mNewAction->setEnabled(true);
+            mOpenInKonquiAction->setEnabled(true);
+            mAlbumImportAction->setEnabled(true);
+        }
+        else
+        {
+            mNewAction->setEnabled(false);
+            mOpenInKonquiAction->setEnabled(false);
+            mAlbumImportAction->setEnabled(false);            
+        }
+        
+        KAction *action;
+        for (action = m_kipiFileActionsImport.first(); action; 
+             action = m_kipiFileActionsImport.next())
+        {
+            action->setEnabled(false);
+        }
+
+        for (action = m_kipiFileActionsExport.first(); action; 
+             action = m_kipiFileActionsExport.next())
+        {
+            action->setEnabled(true);
+        }
     }
 }
 
 void DigikamApp::slot_tagSelected(bool val)
 {
     Album *album = mAlbumManager->currentAlbum();
-    if(album && !album->isRoot())
+    
+    if(!val)
     {
-        mDeleteTagAction->setEnabled(val);
-        mEditTagAction->setEnabled(val);
+        mDeleteTagAction->setEnabled(false);
+        mEditTagAction->setEnabled(false);
+    }
+    else if(!album->isRoot())
+    {
+        mDeleteTagAction->setEnabled(true);
+        mEditTagAction->setEnabled(true);
+        
+        KAction *action;
+        for (action = m_kipiFileActionsImport.first(); action;
+             action = m_kipiFileActionsImport.next())
+        {
+            action->setEnabled(false);
+        }
+
+        for (action = m_kipiFileActionsExport.first(); action;
+             action = m_kipiFileActionsExport.next())
+        {
+            action->setEnabled(true);
+        }
     }
     else
     {
         mDeleteTagAction->setEnabled(false);
         mEditTagAction->setEnabled(false);
+        
+        KAction *action;
+        for (action = m_kipiFileActionsImport.first(); action; 
+             action = m_kipiFileActionsImport.next())
+        {
+            action->setEnabled(false);
+        }
+
+        for (action = m_kipiFileActionsExport.first(); action; 
+             action = m_kipiFileActionsExport.next())
+        {
+            action->setEnabled(true);
+        }
     }
 }
 
@@ -668,7 +892,7 @@ void DigikamApp::slot_gammaAdjustment()
    int ValRet = KApplication::kdeinitExec(QString::fromLatin1("kcmshell"), args, perror, ppid);
 
    if ( ValRet != 0 )
-      KMessageBox::error(0, i18n("Cannot start \"KGamma\" extension from KDE control center;\n"
+      KMessageBox::error(this, i18n("Cannot start \"KGamma\" extension from KDE control center;\n"
                                  "please check your installation."));
 }
 
@@ -677,6 +901,111 @@ void DigikamApp::slot_exit()
     close();
 }
 
+
+QString DigikamApp::convertToLocalUrl( const QString& folder )
+{
+    // This function is copied from k3b.
+    KURL url( folder );
+    if( !url.isLocalFile() ) 
+    {
+#if KDE_IS_VERSION(3,4,91)
+        return KIO::NetAccess::mostLocalURL( url, 0 ).path();
+#else
+#ifndef UDS_LOCAL_PATH
+#define UDS_LOCAL_PATH (72 | KIO::UDS_STRING)
+#else
+        using namespace KIO;
+#endif
+        KIO::UDSEntry e;
+        if( KIO::NetAccess::stat( url, e, 0 ) ) 
+        {
+            const KIO::UDSEntry::ConstIterator end = e.end();
+            for( KIO::UDSEntry::ConstIterator it = e.begin(); it != end; ++it ) 
+            {
+                if( (*it).m_uds == UDS_LOCAL_PATH && !(*it).m_str.isEmpty() )
+                    return KURL::fromPathOrURL( (*it).m_str ).path();
+            }
+        }
+#endif
+     }
+    return url.path();
+}
+
+void DigikamApp::slotDownloadImages( const QString& folder)
+{
+    if (!folder.isNull())
+    {
+        mCameraGuiPath = folder;
+        QTimer::singleShot(0, this, SLOT(slotDownloadImages()));
+    }
+}
+    
+void DigikamApp::slotDownloadImages()
+{
+    if (mCameraGuiPath.isNull())
+            return;
+
+    // Fetch the contents of the device. This is needed to make sure that the 
+    // media:/device gets mounted.
+    // KIO::ListJob *job = KIO::listDir(KURL(mCameraGuiPath), false, false);
+    //
+    // cant use this, as this is not in old kdelibs
+    // KIO::NetAccess::synchronousRun(job,0);
+
+    QString cameraGuiPath = convertToLocalUrl(mCameraGuiPath);
+    kdDebug() << "IN: " << mCameraGuiPath << " OUT: " << cameraGuiPath << endl;
+    
+    if (cameraGuiPath.isNull())
+        return;
+    
+    bool alreadyThere = false;
+    for (uint i = 0 ; i != actionCollection()->count() ; i++)
+    {
+        if (actionCollection()->action(i)->name() == mCameraGuiPath)
+            alreadyThere = true;
+    }
+    
+    if (!alreadyThere)
+    {
+        KAction *cAction  = new KAction(
+                i18n("Browse %1").arg(mCameraGuiPath),
+                "kipi",
+                0,
+                this,
+                SLOT(slotDownloadImages()),
+                actionCollection(),
+                mCameraGuiPath.latin1() );
+
+        mCameraMenuAction->insert(cAction, 0);
+    }
+
+    CameraUI* cgui = new CameraUI(this, 
+                        i18n("Images found in %1").arg(cameraGuiPath),
+                        "directory browse","Fixed", cameraGuiPath);
+    cgui->show();
+    connect(cgui, SIGNAL(signalLastDestination(const KURL&)),
+            mView, SLOT(slotSelectAlbum(const KURL&)));
+    connect(cgui, SIGNAL(signalAlbumSettingsChanged()),
+            SLOT(slotSetupChanged()));
+    connect(cgui, SIGNAL(close()), SLOT(slotCameraUmount()));
+}
+
+void DigikamApp::slotCameraUmount()
+{
+    int items = mMediaItems.count();
+    for (int i=0 ; i != items ; i++)
+    {
+        if (mMediaItems[i].first == mCameraGuiPath && mMediaItems[i].second)
+        {
+            // Ok, item is now mounted and not before, umount it.
+            kdDebug() << "Trying to umount device" << endl;
+            KIO::SimpleJob *job = KIO::unmount(convertToLocalUrl(mCameraGuiPath));
+            // cant use this, as this is not in old kdelibs
+            // KIO::NetAccess::synchronousRun(job,0);
+        }
+    }
+}
+
 void DigikamApp::slotCameraConnect()
 {
     CameraType* ctype = mCameraList->find(QString::fromUtf8(sender()->name()));
@@ -697,7 +1026,7 @@ void DigikamApp::slotCameraAdded(CameraType *ctype)
 {
     if (!ctype) return;
 
-    KAction *cAction = new KAction(ctype->title(), 0,
+    KAction *cAction = new KAction(ctype->title(), "camera", 0,
                                    this, SLOT(slotCameraConnect()),
                                    actionCollection(),
                                    ctype->title().utf8());
@@ -705,6 +1034,65 @@ void DigikamApp::slotCameraAdded(CameraType *ctype)
     ctype->setAction(cAction);
 }
 
+void DigikamApp::slotCameraMediaMenu()
+{
+    mMediaItems.clear();
+    
+    mCameraMediaList->clear();
+    mCameraMediaList->insertItem(i18n("No Media Devices Found"),1);
+    mCameraMediaList->setItemEnabled(1,false);
+        
+    KURL kurl("media:/");
+    KIO::ListJob *job = KIO::listDir(kurl, false, false);
+    connect( job, SIGNAL(entries(KIO::Job*,const KIO::UDSEntryList&)),
+             SLOT(slotCameraMediaMenuEntries(KIO::Job*,const KIO::UDSEntryList&)) );
+}
+
+void DigikamApp::slotCameraMediaMenuEntries( Job *, const UDSEntryList & list )
+{
+    int i = 0;
+
+    for(KIO::UDSEntryList::ConstIterator it = list.begin() ; it != list.end(); ++it)
+    {
+        QString name;
+        QString path;
+        bool unmounted = false;
+
+        for ( UDSEntry::const_iterator et = (*it).begin() ; et !=   (*it).end() ; ++ et ) 
+        {
+            if ( (*et).m_uds == KIO::UDS_NAME)
+                name = ( *et ).m_str;
+
+            if ( (*et).m_uds == KIO::UDS_URL)
+                path = ( *et ).m_str;
+
+            if ( (*et).m_uds == KIO::UDS_MIME_TYPE &&
+                   ( (*et).m_str == "media/removable_unmounted" || 
+                     (*et).m_str == "media/camera_unmounted" ) )
+                unmounted=true;
+
+            //kdDebug() << ( *et ).m_str << unmounted << endl;
+       }
+
+       if (!name.isEmpty() && !path.isEmpty())
+       {
+            if (i==0)
+                mCameraMediaList->clear();
+            
+            mMediaItems[i] = qMakePair(path,unmounted);
+            
+            mCameraMediaList->insertItem(name, this, SLOT(slotDownloadImagesFromMedia(int)), 0, i);
+            mCameraMediaList->setItemParameter(i, i);
+            i++;
+       }
+    }
+}
+
+void DigikamApp::slotDownloadImagesFromMedia( int id )
+{
+    slotDownloadImages( mMediaItems[id].first );
+}
+
 void DigikamApp::slotCameraRemoved(CameraType *ctype)
 {
     if (!ctype) return;
@@ -718,14 +1106,14 @@ void DigikamApp::slotCameraRemoved(CameraType *ctype)
 void DigikamApp::slotCameraAutoDetect()
 {
     bool retry = false;
-    
+
     CameraType* ctype = mCameraList->autoDetect(retry);
     if (!ctype && retry)
     {
         QTimer::singleShot(0, this, SLOT(slotCameraAutoDetect()));
         return;
     }
-    
+
     if (ctype && ctype->action())
     {
         ctype->action()->activate();
@@ -770,11 +1158,16 @@ void DigikamApp::slotSetupCamera()
 
 void DigikamApp::slotSetupChanged()
 {
+    if(mAlbumSettings->getAlbumLibraryPath() != mAlbumManager->getLibraryPath())
+        mView->clearHistory();
     mAlbumManager->setLibraryPath(mAlbumSettings->getAlbumLibraryPath());
+    mAlbumManager->startScan();
+
     mView->applySettings(mAlbumSettings);
     updateDeleteTrashMenu();
     if (ImageWindow::imagewindow())
         ImageWindow::imagewindow()->applySettings();
+
     m_config->sync();
 }
 
@@ -802,9 +1195,9 @@ void DigikamApp::slotConfToolbars()
     saveMainWindowSettings(KGlobal::config());
     KEditToolbar *dlg = new KEditToolbar(actionCollection(), "digikamui.rc");
 
-    if (dlg->exec())
+    if(dlg->exec());
     {
-        createGUI("digikamui.rc");
+        createGUI(QString::fromLatin1( "digikamui.rc" ), false);
         applyMainWindowSettings(KGlobal::config());
         plugActionList( QString::fromLatin1("file_actions_import"), m_kipiFileActionsImport );
         plugActionList( QString::fromLatin1("image_actions"), m_kipiImageActions );
@@ -813,7 +1206,6 @@ void DigikamApp::slotConfToolbars()
         plugActionList( QString::fromLatin1("album_actions"), m_kipiAlbumActions );
         plugActionList( QString::fromLatin1("file_actions_export"), m_kipiFileActionsExport );
     }
-
     delete dlg;
 }
 
@@ -869,6 +1261,11 @@ void DigikamApp::loadPlugins()
 
     KipiPluginLoader_->loadPlugins();
 
+    KipiInterface_->slotCurrentAlbumChanged(mAlbumManager->currentAlbum());
+
+    // Setting the initial menu options after all plugins have been loaded
+    mView->slot_albumSelected(mAlbumManager->currentAlbum());
+
     m_ImagePluginsLoader = new ImagePluginLoader(this, mSplash);
 }
 
@@ -888,10 +1285,8 @@ void DigikamApp::slotKipiPluginPlug()
     m_kipiBatchActions.clear();
     m_kipiAlbumActions.clear();
 
-    m_kipiAlbumActions.append( mAddImagesAction );
-
     KIPI::PluginLoader::PluginList list = KipiPluginLoader_->pluginList();
-    
+
     int cpt = 0;
 
     for( KIPI::PluginLoader::PluginList::Iterator it = list.begin() ; it != list.end() ; ++it )
@@ -902,7 +1297,7 @@ void DigikamApp::slotKipiPluginPlug()
             continue;
 
         ++cpt;
-        
+
         if(mSplash)
             mSplash->message(i18n("Loading: %1").arg((*it)->name()));
 
@@ -943,10 +1338,10 @@ void DigikamApp::slotKipiPluginPlug()
 
         plugin->actionCollection()->readShortcutSettings();
     }
-    
+
     if(mSplash)
         mSplash->message(i18n("1 Kipi Plugin Loaded", "%n Kipi Plugins Loaded", cpt));
-    
+
     // Create GUI menu in according with plugins.
 
     plugActionList( QString::fromLatin1("file_actions_export"), m_kipiFileActionsExport );
@@ -960,7 +1355,13 @@ void DigikamApp::slotKipiPluginPlug()
 void DigikamApp::loadCameras()
 {
     mCameraList->load();
+  
+//    TA: unfinished  
+//    mCameraMenuAction->popupMenu()->insertSeparator();
+//    mCameraMenuAction->popupMenu()->insertItem(i18n("Media Browse"), mCameraMediaList);
+    
     mCameraMenuAction->popupMenu()->insertSeparator();
+    
     mCameraMenuAction->insert(new KAction(i18n("Add Camera..."), 0,
                                           this, SLOT(slotSetupCamera()),
                                           actionCollection(),
@@ -969,21 +1370,21 @@ void DigikamApp::loadCameras()
 
 void DigikamApp::populateThemes()
 {
-    ThemeEngine::instance()->scanThemes();
-    QStringList themes(ThemeEngine::instance()->themeNames());
+    Digikam::ThemeEngine::instance()->scanThemes();
+    QStringList themes(Digikam::ThemeEngine::instance()->themeNames());
 
     mThemeMenuAction->setItems(themes);
     int index = themes.findIndex(mAlbumSettings->getCurrentTheme());
     if (index == -1)
         index = themes.findIndex(i18n("Default"));
     mThemeMenuAction->setCurrentItem(index);
-    ThemeEngine::instance()->slotChangeTheme(mThemeMenuAction->currentText());
+    Digikam::ThemeEngine::instance()->slotChangeTheme(mThemeMenuAction->currentText());
 }
 
 void DigikamApp::slotChangeTheme(const QString& theme)
 {
     mAlbumSettings->setCurrentTheme(theme);
-    ThemeEngine::instance()->slotChangeTheme(theme);
+    Digikam::ThemeEngine::instance()->slotChangeTheme(theme);
 }
 
 void DigikamApp::updateDeleteTrashMenu()
@@ -1004,6 +1405,12 @@ void DigikamApp::updateDeleteTrashMenu()
     }
 }
 
+void DigikamApp::slotDatabaseRescan()
+{
+    ScanLib sLib;
+    sLib.startScan();
+}
+
 DigikamApp* DigikamApp::m_instance = 0;
 
 #include "digikamapp.moc"
