@@ -1,5 +1,4 @@
 /* ============================================================
- * File  : cameraui.cpp
  * Author: Renchi Raju <renchi@pooh.tam.uiuc.edu>
  * Date  : 2004-09-16
  * Description : 
@@ -8,7 +7,7 @@
 
  * This program is free software; you can redistribute it
  * and/or modify it under the terms of the GNU General
- * Public License as published bythe Free Software Foundation;
+ * Public License as published by the Free Software Foundation;
  * either version 2, or (at your option)
  * any later version.
  * 
@@ -35,6 +34,8 @@
 #include <qlineedit.h>
 #include <qprogressbar.h>
 #include <qtimer.h>
+#include <qfile.h>
+#include <qfileinfo.h>
 
 #include <kmessagebox.h>
 #include <kglobal.h>
@@ -45,15 +46,21 @@
 #include <kiconloader.h>
 #include <kpopupmenu.h>
 #include <khelpmenu.h>
+#include <kdebug.h>
 
-#include <kdeversion.h>
-#if KDE_IS_VERSION(3,2,0)
 #include <kcalendarsystem.h>
-#endif
+
+extern "C"
+{
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
+}
 
 #include "albummanager.h"
+#include "albumsettings.h"
 #include "album.h"
-#include "dirselectdialog.h"
+#include "albumselectdialog.h"
 #include "renamecustomizer.h"
 #include "animwidget.h"
 #include "gpiteminfodlg.h"
@@ -61,11 +68,13 @@
 #include "cameraiconitem.h"
 #include "cameracontroller.h"
 #include "cameraui.h"
+#include "scanlib.h"
 
-CameraUI::CameraUI(QWidget* parent, const QString& title,
+CameraUI::CameraUI(QWidget* /*parent*/, const QString& title,
                    const QString& model, const QString& port,
                    const QString& path)
-    : QDialog(parent, 0, false, WDestructiveClose)    
+    : QDialog(0, // B.K.O # 116485: no parent for this modal dialog.
+              0, false, WDestructiveClose)    
 {
     // -- setup view -----------------------------------------
 
@@ -85,11 +94,11 @@ CameraUI::CameraUI(QWidget* parent, const QString& title,
     viewBoxLayout->addMultiCellWidget(m_view, 0, 0, 0, 3);
 
     m_cancelBtn = new QToolButton(viewBox);
-    QPixmap icon = kapp->iconLoader()->loadIcon("stop",
-                                                KIcon::NoGroup,
-                                                22);
+    QIconSet iconSet = kapp->iconLoader()->loadIconSet("stop",
+                                                       KIcon::Toolbar,
+                                                       22);
     m_cancelBtn->setText(i18n("Cancel"));
-    m_cancelBtn->setIconSet(icon);
+    m_cancelBtn->setIconSet(iconSet);
     m_cancelBtn->setEnabled(false);
     viewBoxLayout->addWidget(m_cancelBtn, 1, 0);
     
@@ -104,9 +113,13 @@ CameraUI::CameraUI(QWidget* parent, const QString& title,
     mainLayout->addWidget(viewBox);
 
     QHBoxLayout* btnLayout = new QHBoxLayout();
+
+    QPushButton* helpBtn;
+    QPushButton* selectBtn;
     
     m_advBtn      = new QPushButton(i18n("&Advanced %1").arg(">>"), this);
-    m_helpBtn     = new QPushButton(i18n("&Help"), this);
+    helpBtn       = new QPushButton(i18n("&Help"), this);
+    selectBtn     = new QPushButton(i18n("&Select"), this);
     m_downloadBtn = new QPushButton(i18n("&Download"), this);
     m_deleteBtn   = new QPushButton(i18n("D&elete"), this);
     m_closeBtn    = new QPushButton(i18n("&Close"), this);
@@ -114,19 +127,12 @@ CameraUI::CameraUI(QWidget* parent, const QString& title,
     btnLayout->addWidget(m_advBtn);
     btnLayout->addItem(new QSpacerItem(10,10,QSizePolicy::Expanding,
                                        QSizePolicy::Fixed));
-    btnLayout->addWidget(m_helpBtn);
+    btnLayout->addWidget(helpBtn);
+    btnLayout->addWidget(selectBtn);
     btnLayout->addWidget(m_downloadBtn);
     btnLayout->addWidget(m_deleteBtn);
     btnLayout->addWidget(m_closeBtn);
 
-    // About popupmenu button using a slot for calling the camera interface
-    // anchor in Digikam handbook.
-    
-    KHelpMenu* helpMenu = new KHelpMenu(this, KApplication::kApplication()->aboutData(), false);
-    helpMenu->menu()->removeItemAt(0);
-    helpMenu->menu()->insertItem(i18n("Digikam Handbook"), this, SLOT(slotHelp()), 0, -1, 0);
-    m_helpBtn->setPopup( helpMenu->menu() );
-
     mainLayout->addLayout(btnLayout);
     
     m_advBox = new QVBox(this);
@@ -137,13 +143,44 @@ CameraUI::CameraUI(QWidget* parent, const QString& title,
     m_renameCustomizer = new RenameCustomizer(m_advBox);
     m_view->setRenameCustomizer(m_renameCustomizer);
 
-    QVGroupBox* rotateBox = new QVGroupBox(i18n("Auto Orient"), m_advBox);
+    QVGroupBox* exifBox = new QVGroupBox(i18n("Use Camera-provided Information (EXIF)"),
+                                         m_advBox);
     m_autoRotateCheck = new QCheckBox(i18n("Automatically rotate/flip using "
-                                           "camera provided information (EXIF)"),
-                                      rotateBox);
+                                           "camera-provided information (EXIF)"),
+                                      exifBox);
+    m_autoAlbumCheck = new QCheckBox(i18n("Download photos into automatically "
+                                          "created date-based sub-albums of "
+                                          "destination album"),
+                                      exifBox);
     
     mainLayout->addWidget(m_advBox);
+
+    // About popupmenu button using a slot for calling the camera interface
+    // anchor in Digikam handbook.
     
+    KHelpMenu* helpMenu = new KHelpMenu(this, KApplication::kApplication()->aboutData(),
+                                        false);
+    helpMenu->menu()->removeItemAt(0);
+    helpMenu->menu()->insertItem(i18n("Digikam Handbook"),
+                                 this, SLOT(slotHelp()), 0, -1, 0);
+    helpBtn->setPopup( helpMenu->menu() );
+
+    // -------------------------------------------------------------------------
+    
+    QPopupMenu* selectMenu = new QPopupMenu(this);
+    selectMenu->insertItem(i18n("Select &All"),
+                           m_view, SLOT(slotSelectAll()));
+    selectMenu->insertItem(i18n("Select N&one"),
+                           m_view, SLOT(slotSelectNone()));
+    selectMenu->insertItem(i18n("&Invert Selection"),
+                           m_view, SLOT(slotSelectInvert()));
+    selectMenu->insertSeparator();
+    selectMenu->insertItem(i18n("Select &New Items"),
+                           m_view, SLOT(slotSelectNew()));
+    selectBtn->setPopup(selectMenu);    
+
+    // -------------------------------------------------------------------------
+        
     m_downloadMenu = new QPopupMenu(this);
     m_downloadMenu->insertItem(i18n("Download Selected"),
                                this, SLOT(slotDownloadSelected()),
@@ -154,6 +191,8 @@ CameraUI::CameraUI(QWidget* parent, const QString& title,
     m_downloadMenu->setItemEnabled(0, false);
     m_downloadBtn->setPopup(m_downloadMenu);
 
+    // -------------------------------------------------------------------------
+    
     m_deleteMenu = new QPopupMenu(this);
     m_deleteMenu->insertItem(i18n("Delete Selected"),
                              this, SLOT(slotDeleteSelected()),
@@ -164,6 +203,8 @@ CameraUI::CameraUI(QWidget* parent, const QString& title,
     m_deleteMenu->setItemEnabled(0, false);
     m_deleteBtn->setPopup(m_deleteMenu);
 
+    // -------------------------------------------------------------------------
+    
     connect(m_closeBtn, SIGNAL(clicked()),
             SLOT(close()));
     connect(m_advBtn, SIGNAL(clicked()),
@@ -231,9 +272,27 @@ bool CameraUI::isBusy() const
 
 void CameraUI::closeEvent(QCloseEvent* e)
 {
+    // When a directory is created, a watch is put on it to spot new files
+    // but it can occur that the file is copied there before the watch is
+    // completely setup. That is why as an extra safeguard run scanlib
+    // over the folders we used. Bug: 119201
+    ScanLib sLib;
+    for (QStringList::iterator it = m_foldersToScan.begin(); 
+         it != m_foldersToScan.end(); ++it)
+    {
+        kdDebug() << "Scanning " << (*it) << endl;
+        sLib.findMissingItems( (*it) );
+    }
+    
+    //---------------------------------------------------
+
+    if(!m_lastDestURL.isEmpty())
+        emit signalLastDestination(m_lastDestURL);
+
+    emit close();
     delete m_controller;
     saveSettings();
-    e->accept();    
+    e->accept();
 }
 
 
@@ -280,7 +339,10 @@ void CameraUI::slotConnected(bool val)
                                     i18n("Failed to connect to camera. "
                                          "Please make sure its connected "
                                          "properly and turned on. "
-                                         "Would you like to try again?"))
+                                         "Would you like to try again?"), 
+                                    i18n("Connection Failed"),
+                                    i18n("Retry"),
+                                    i18n("Abort"))
           == KMessageBox::Yes)
           QTimer::singleShot(0, m_controller, SLOT(slotConnect()));
       else
@@ -339,103 +401,27 @@ void CameraUI::slotErrorMsg(const QString& msg)
 
 void CameraUI::slotDownloadSelected()
 {
-    AlbumManager* man = AlbumManager::instance();
-    QString libPath(man->getLibraryPath());
-    QString currPath;
-
-    Album* album = man->currentAlbum();
-    if (!album || album->type() != Album::PHYSICAL)
-        currPath = libPath;
-    else
-    {
-        currPath = ((PAlbum*)album)->getKURL().path();
-    }
-
-    QString header(i18n("Select Destination Album for "
-                        "Importing Camera Images"));
-
-    QString newDirName;
-    QIconViewItem* firstItem = m_view->firstItem();
-    if (firstItem)
-    {
-        CameraIconViewItem* iconItem =
-            static_cast<CameraIconViewItem*>(firstItem);
-        
-        QDateTime date;
-        date.setTime_t(iconItem->itemInfo()->mtime);
-#if KDE_IS_VERSION(3,2,0)
-        newDirName = QString("%1, %2, %3")
-                     .arg(KGlobal::locale()->calendar()->year(date.date()))
-                     .arg(KGlobal::locale()->calendar()->monthName(date.date()))
-                     .arg(KGlobal::locale()->calendar()->day(date.date()));
-#else
-        newDirName = QString("%1, %2, %3")
-                     .arg(date.date().year())
-                     .arg(KGlobal::locale()->monthName(date.date().month()))
-                     .arg(date.date().day());
-#endif
-    }
-    
-    KURL url = DirSelectDialog::selectDir(libPath, currPath,
-                                          this, header, newDirName);
-    if (!url.isValid())
-        return;
-    
-    m_controller->downloadPrep();
-
-    QString downloadName;
-    QString name;
-    QString folder;
-    bool    autoRotate;
-
-    autoRotate = m_autoRotateCheck->isChecked();
-
-    int total = 0;
-    for (QIconViewItem* item = m_view->firstItem(); item;
-         item = item->nextItem())
-    {
-        CameraIconViewItem* iconItem = static_cast<CameraIconViewItem*>(item);
-        if (iconItem->isSelected())
-        {
-            folder = iconItem->m_itemInfo->folder;
-            name   = iconItem->m_itemInfo->name;
-            downloadName = iconItem->getDownloadName();
-
-            KURL u(url);
-            u.addPath(downloadName.isEmpty() ? name : downloadName);
-            
-            m_controller->download(folder, name, u.path(), autoRotate);
-            total++;
-        }
-    }
-
-    if (total <= 0)
-        return;
-    
-    m_progress->setProgress(0);
-    m_progress->setTotalSteps(total);
-    m_progress->show();
+    slotDownload(true);
 }
 
 void CameraUI::slotDownloadAll()
+{
+    slotDownload(false);
+}
+
+void CameraUI::slotDownload(bool onlySelected)
 {
     AlbumManager* man = AlbumManager::instance();
-    QString libPath(man->getLibraryPath());
-    QString currPath;
 
     Album* album = man->currentAlbum();
-    if (!album || album->type() != Album::PHYSICAL)
-        currPath = libPath;
-    else
-    {
-        currPath = ((PAlbum*)album)->getKURL().path();
-    }
+    if (album && album->type() != Album::PHYSICAL)
+        album = 0;
 
     QString header(i18n("Select Destination Album for "
                         "Importing Camera Images"));
 
     QString newDirName;
-    QIconViewItem* firstItem = m_view->firstItem();
+    IconItem* firstItem = m_view->firstItem();
     if (firstItem)
     {
         CameraIconViewItem* iconItem =
@@ -443,52 +429,81 @@ void CameraUI::slotDownloadAll()
         
         QDateTime date;
         date.setTime_t(iconItem->itemInfo()->mtime);
-#if KDE_IS_VERSION(3,2,0)
         newDirName = QString("%1, %2, %3")
                      .arg(KGlobal::locale()->calendar()->year(date.date()))
                      .arg(KGlobal::locale()->calendar()->monthName(date.date()))
                      .arg(KGlobal::locale()->calendar()->day(date.date()));
-#else
-        newDirName = QString("%1, %2, %3")
-                     .arg(date.date().year())
-                     .arg(KGlobal::locale()->monthName(date.date().month()))
-                     .arg(date.date().day());
-#endif
     }
-    
-    KURL url = DirSelectDialog::selectDir(libPath, currPath,
-                                          this, header, newDirName);
-    if (!url.isValid())
+
+
+    album = AlbumSelectDialog::selectAlbum(this,
+                                           (PAlbum*)album,
+                                           header,
+                                           newDirName,
+                                           m_autoAlbumCheck->isChecked());
+
+    if (!album)
         return;
+
+    KURL url;
+    url.setPath(((PAlbum*)album)->folderPath());
     
     m_controller->downloadPrep();
 
     QString downloadName;
     QString name;
     QString folder;
+    time_t  mtime;
     bool    autoRotate;
+    bool    autoAlbum;
 
     autoRotate = m_autoRotateCheck->isChecked();
+    autoAlbum  = m_autoAlbumCheck->isChecked();
 
     int total = 0;
-    for (QIconViewItem* item = m_view->firstItem(); item;
+    for (IconItem* item = m_view->firstItem(); item;
          item = item->nextItem())
     {
+        if (onlySelected && !(item->isSelected()))
+            continue;
+
         CameraIconViewItem* iconItem = static_cast<CameraIconViewItem*>(item);
         folder = iconItem->m_itemInfo->folder;
         name   = iconItem->m_itemInfo->name;
         downloadName = iconItem->getDownloadName();
+        mtime        = iconItem->m_itemInfo->mtime;
         
         KURL u(url);
-        u.addPath(downloadName.isEmpty() ? name : downloadName);
-            
+        if (autoAlbum)
+        {
+            QDateTime date;
+            date.setTime_t(mtime);
+            QString dirName(date.toString("yyyy-MM-dd"));
+            QString errMsg;
+            if (!createAutoAlbum(url, dirName, date.date(), errMsg))
+            {
+                KMessageBox::error(this, errMsg);
+                return;
+            }
+
+            u.addPath(dirName);
+            m_foldersToScan.append(u.path());
+            u.addPath(downloadName.isEmpty() ? name : downloadName);
+        }
+        else
+        {
+            m_foldersToScan.append(u.path());
+            u.addPath(downloadName.isEmpty() ? name : downloadName);
+        }
         m_controller->download(folder, name, u.path(), autoRotate);
+        addFileExtension(QFileInfo(u.path()).extension(false));
         total++;
     }
 
     if (total <= 0)
         return;
     
+    m_lastDestURL = url;            
     m_progress->setProgress(0);
     m_progress->setTotalSteps(total);
     m_progress->show();
@@ -500,7 +515,7 @@ void CameraUI::slotDeleteSelected()
     QStringList files;
     QStringList deleteList;
     
-    for (QIconViewItem* item = m_view->firstItem(); item;
+    for (IconItem* item = m_view->firstItem(); item;
          item = item->nextItem())
     {
         CameraIconViewItem* iconItem = static_cast<CameraIconViewItem*>(item);
@@ -545,7 +560,7 @@ void CameraUI::slotDeleteAll()
     QStringList files;
     QStringList deleteList;
     
-    for (QIconViewItem* item = m_view->firstItem(); item;
+    for (IconItem* item = m_view->firstItem(); item;
          item = item->nextItem())
     {
         CameraIconViewItem* iconItem = static_cast<CameraIconViewItem*>(item);
@@ -605,8 +620,14 @@ void CameraUI::slotItemsSelected(bool selected)
     m_deleteMenu->setItemEnabled(0, selected);
 }
 
-void CameraUI::slotDownloaded(const QString&, const QString&)
+void CameraUI::slotDownloaded(const QString& folder, const QString& file)
 {
+    CameraIconViewItem* iconItem = m_view->findItem(folder, file);
+    if (iconItem)
+    {
+        iconItem->setDownloaded();
+    }
+    
     int curr = m_progress->progress();
     m_progress->setProgress(curr+1);
 }
@@ -632,6 +653,7 @@ void CameraUI::readSettings()
     w = config->readNumEntry("Width", 500);
     h = config->readNumEntry("Height", 500);
     m_autoRotateCheck->setChecked(config->readBoolEntry("AutoRotate", true));
+    m_autoAlbumCheck->setChecked(config->readBoolEntry("AutoAlbum", false));
 
     resize(w, h);
 }
@@ -644,7 +666,63 @@ void CameraUI::saveSettings()
     config->writeEntry("Width", width());
     config->writeEntry("Height", height());
     config->writeEntry("AutoRotate", m_autoRotateCheck->isChecked());
+    config->writeEntry("AutoAlbum", m_autoAlbumCheck->isChecked());
     config->sync();
 }
 
+bool CameraUI::createAutoAlbum(const KURL& parentURL,
+                               const QString& name,
+                               const QDate& date,
+                               QString& errMsg)
+{
+    KURL u(parentURL);
+    u.addPath(name);
+
+    // first stat to see if the album exists
+    struct stat buf;
+    if (::stat(QFile::encodeName(u.path()), &buf) == 0)
+    {
+        // now check if its really a directory
+        if (S_ISDIR(buf.st_mode))
+            return true;
+        else
+        {
+            errMsg = i18n("A file with same name (%1) exists in folder %2")
+                     .arg(name)
+                     .arg(parentURL.path());
+            return false;
+        }
+    }
+
+    // looks like the directory does not exist, try to create it
+
+    AlbumManager* aman = AlbumManager::instance();
+    PAlbum* parent =  aman->findPAlbum(parentURL);
+    if (!parent)
+    {
+        errMsg = i18n("Failed to find Album for path '%1'")
+                 .arg(parentURL.path());
+        return false;
+    }
+
+    return aman->createPAlbum(parent, name, QString(""),
+                              date, QString(""), errMsg);
+}
+
+void CameraUI::addFileExtension(const QString& ext)
+{
+    AlbumSettings* settings = AlbumSettings::instance();
+    if (!settings)
+        return;
+
+    if (settings->getImageFileFilter().upper().contains(ext.upper()) ||
+        settings->getMovieFileFilter().upper().contains(ext.upper()) ||
+        settings->getAudioFileFilter().upper().contains(ext.upper()) ||
+        settings->getRawFileFilter().upper().contains(ext.upper()))
+        return;
+
+    settings->setImageFileFilter(settings->getImageFileFilter() + QString(" *.") + ext);
+    emit signalAlbumSettingsChanged();
+}
+
 #include "cameraui.moc"
