@@ -19,13 +19,14 @@
  * 
  * ============================================================ */
 
-#define PI 3.14159265
- 
 // Lib Tiff includes.
 
 extern "C" 
 {
 #include <tiffio.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
 }
  
 // C++ includes.
@@ -33,6 +34,7 @@ extern "C"
 #include <cmath>
 #include <cstdio>
 #include <cstdlib>
+#include <iostream>
 
 // Qt includes.
 
@@ -46,7 +48,9 @@ extern "C"
 
 // KDE includes.
 
+#include <kfilemetainfo.h>
 #include <kdebug.h>
+#include <kprocess.h>
 #include <libkexif/kexifdata.h>
 
 // Imlib2 includes.
@@ -56,16 +60,19 @@ extern "C"
 
 // Local includes.
 
+#include "dcraw_parse.h"
 #include "undomanager.h"
 #include "undoaction.h"
 #include "imlibinterface.h"
+#include "dcrawbinary.h"
 
 namespace Digikam
 {
 
 #define MaxRGB 255L
 
-class ImlibInterfacePrivate {
+class ImlibInterfacePrivate
+{
 
 public:
 
@@ -98,6 +105,7 @@ public:
     Imlib_Color_Modifier cmod;
     Imlib_Load_Error     errorRet;
     QString              filename;
+    mode_t               filePermissions;
 
     UndoManager*         undoMan;
 };
@@ -163,47 +171,221 @@ ImlibInterface::~ImlibInterface()
     m_instance = 0;
 }
 
-bool ImlibInterface::load(const QString& filename)
+int ImlibInterface::fileFormat(const QString& filePath)
+{
+    if ( filePath == QString::null )
+        return NONE_IMAGE;
+
+    // In first we trying to check the file extension. This is mandatory because
+    // some tiff files are detected like RAW files by dcraw::parse method.
+
+    QFileInfo fileInfo(filePath);
+    if (!fileInfo.exists())
+    {
+        kdDebug() << k_funcinfo << "Failed to open file" << endl;
+        return NONE_IMAGE;
+    }
+    
+    QString ext = fileInfo.extension().upper();
+
+    if (ext == QString("JPEG") || ext == QString("JPG"))
+        return JPEG_IMAGE;
+    else if (ext == QString("PNG"))
+        return PNG_IMAGE;
+    else if (ext == QString("TIFF") || ext == QString("TIF"))
+        return TIFF_IMAGE;
+
+    // In second, we trying to parse file header.
+        
+    FILE* f = fopen(QFile::encodeName(filePath), "rb");
+    
+    if (!f)
+    {
+        kdWarning() << "Failed to open file" << endl;
+        return NONE_IMAGE;
+    }
+
+    const int headerLen = 8;
+    unsigned char header[headerLen];
+    
+    if (fread(&header, 8, 1, f) != 1)
+    {
+        kdWarning() << "Failed to read header" << endl;
+        fclose(f);
+        return NONE_IMAGE;
+    }
+
+    fclose(f);
+    
+    DcrawParse rawFileParser;
+    uchar jpegID[2]    = { 0xFF, 0xD8 };   
+    uchar tiffBigID[2] = { 0x4D, 0x4D };
+    uchar tiffLilID[2] = { 0x49, 0x49 };
+    uchar pngID[8]     = { 0x89, 0x50, 0x4E, 0x47, 0x0D, 0x0A, 0x1A, 0x0A };
+
+    if (memcmp(&header, &jpegID, 2) == 0)            // JPEG file ?
+    {
+        return JPEG_IMAGE;
+    }
+    else if (memcmp(&header, &pngID, 8) == 0)        // PNG file ?
+    {
+        return PNG_IMAGE;
+    }
+    else if (rawFileParser.getCameraModel( QFile::encodeName(filePath), NULL, NULL) == 0)
+    {
+        // RAW File test using dcraw.  
+        // Need to test it before TIFF because any RAW file 
+        // formats using TIFF header.
+        return RAW_IMAGE;
+    }
+    else if (memcmp(&header, &tiffBigID, 2) == 0 ||  // TIFF file ?
+             memcmp(&header, &tiffLilID, 2) == 0)
+    {
+        return TIFF_IMAGE;
+    }
+    
+    // In others cases, QImage will be used to try to open file.
+    return QIMAGE_IMAGE;
+
+}
+
+bool ImlibInterface::load(const QString& filename, bool *isReadOnly)
 {
     bool valRet;
-    d->valid = false;
+    *isReadOnly = false;
+    d->valid    = false;
 
     d->filename = filename;
     
     imlib_context_push(d->context);
-    if (d->image) {
+    if (d->image) 
+    {
         imlib_context_set_image(d->image);
         imlib_free_image();
         d->image = 0;
     }
 
-    d->width      = 0;
-    d->height     = 0;
-    d->origWidth  = 0;
-    d->origHeight = 0;
-    d->selX       = 0;
-    d->selY       = 0;
-    d->selW       = 0;
-    d->selH       = 0;
-    d->gamma      = 1.0;
-    d->contrast   = 1.0;
-    d->brightness = 0.0;
+    d->width           = 0;
+    d->height          = 0;
+    d->origWidth       = 0;
+    d->origHeight      = 0;
+    d->selX            = 0;
+    d->selY            = 0;
+    d->selW            = 0;
+    d->selH            = 0;
+    d->gamma           = 1.0;
+    d->contrast        = 1.0;
+    d->brightness      = 0.0;
+    d->filePermissions = 0;
 
     imlib_context_set_color_modifier(d->cmod);
     imlib_reset_color_modifier();
+    Imlib_Load_Error errorReturn;
 
     d->undoMan->clear();
-    
-    Imlib_Load_Error errorReturn;
-    d->image = imlib_load_image_with_error_return(QFile::encodeName(filename),
-                                                  &errorReturn);
 
-    // try to load with kde imageio
-    if (!d->image)
+    switch (fileFormat(filename))
     {
-        QImage qtimage(filename);
-        if (!qtimage.isNull())
+        case NONE_IMAGE:
         {
+            kdWarning() << filename << " : Unknown image format !!!" << endl;
+            return false;
+            break;
+        }    
+        
+        case JPEG_IMAGE:
+        case PNG_IMAGE:
+        {
+            // Try to load image using imlib2.
+            
+            d->image = imlib_load_image_with_error_return(QFile::encodeName(filename), &errorReturn);
+        
+            if (!d->image)
+            {
+                kdWarning() << filename << " : Failed to load image using imlib2: error number: "
+                            << errorReturn << endl;
+            }
+            
+            break;
+        }
+        
+        case RAW_IMAGE:
+        {
+            // Try to load using dcraw program.
+    
+            QCString command;
+            
+            // run dcraw with options:
+            // -c : write to stdout
+            // -q : Use simple bilinear interpolation for quick results
+            // -2 : 8bit ppm output
+            // -w : Use camera white balance, if possible  
+            // -a : Use automatic white balance
+            command  = DcrawBinary::instance()->path();
+            command += " -c -2 -w -a -q 0 ";
+            command += QFile::encodeName( KProcess::quote( filename ) );
+            kdWarning() << "Running dcraw command : " << command << endl;
+        
+            FILE* f = popen( command.data(), "r" );
+        
+            if ( !f )
+            {
+                kdWarning() << "dcraw program unvailable." << endl;
+                break;
+            }
+            else
+            {
+                int  width, height, rgbmax;
+                char nl;
+            
+                if (fscanf (f, "P6 %d %d %d%c", &width, &height, &rgbmax, &nl) != 4) 
+                {
+                    kdWarning() << "Not a raw digital camera image." << endl;
+                    pclose (f);
+                    break;
+                }
+            
+                d->image = imlib_create_image(width, height);
+                imlib_context_set_image(d->image);
+                DATA32* data = imlib_image_get_data_for_reading_only();
+                uchar s[3];
+                
+                for (int i = 0; i < width*height; i++)
+                {
+                    fread (s, 3 *sizeof(unsigned char), 1, f);
+            
+                    // Swap byte order to preserve compatibility with PPC.
+                
+                    if (QImage::systemByteOrder() == QImage::BigEndian)
+                        swab((const char *) s, (char *) s, 3 *sizeof(unsigned char));
+            
+                    // No need to adapt RGB components accordinly with rgbmax value because dcraw
+                    // always return rgbmax to 255 in 8 bits/color/pixels.
+                    
+                    *data = 0xFF000000 | (s[0] << 16) | (s[1] << 8) | s[2];
+    
+                    data++;
+                }
+            
+                pclose( f );
+                *isReadOnly = true;
+            }
+                
+            break;
+        }
+        
+        default:
+        {
+            // Try to load with kde imageio
+            
+            QImage qtimage(filename);
+            
+            if (qtimage.isNull())             
+            {
+                kdWarning() << filename<< " : Failed to load image using kimgio." << endl;
+                break;
+            }
+               
             if (!qtimage.depth() == 32)
                 qtimage.convertDepth(32);
 
@@ -211,14 +393,23 @@ bool ImlibInterface::load(const QString& filename)
             imlib_context_set_image(d->image);
             DATA32* data = imlib_image_get_data_for_reading_only();
             memcpy(data, qtimage.bits(), qtimage.numBytes());
-            kdDebug() << "Loaded image with kde imageio resources: "
-                      << filename
-                      << ", width="  << qtimage.width()
-                      << ", height=" << qtimage.height() << endl;
+            kdWarning() << "Loaded image with kde imageio resources: " 
+                        << filename.ascii()
+                        << ", width="  << qtimage.width()
+                        << ", height=" << qtimage.height() << endl;
+            
+            break;
         }
     }
 
-    if (d->image) {
+    if (d->image)
+    {
+        struct stat stbuf;
+        if (::stat(QFile::encodeName(filename), &stbuf) == 0)
+        {
+            d->filePermissions = stbuf.st_mode;
+        }
+        
         imlib_context_set_image(d->image);
         imlib_image_set_changes_on_disk();
 
@@ -231,9 +422,8 @@ bool ImlibInterface::load(const QString& filename)
 
         valRet = true;
     }
-    else {
-        kdWarning() << k_funcinfo << "Failed to load image: error number: "
-                    << errorReturn << endl;
+    else 
+    {
         valRet = false;
     }
         
@@ -256,7 +446,7 @@ bool ImlibInterface::exifRotated()
 
 void ImlibInterface::exifRotate(const QString& filename)
 {
-   // Rotate image based on EXIF rotate tag
+    // Rotate image based on EXIF rotate tag
     KExifData exifData;
 
     if(!exifData.readFromFile(filename))
@@ -333,20 +523,33 @@ void ImlibInterface::undo()
 {
     if (!d->undoMan->anyMoreUndo())
     {
-        emit signalModified(false);
+        emit signalModified(false, d->undoMan->anyMoreRedo());
         return;
     }
 
     d->undoMan->undo();
-    emit signalModified(d->undoMan->anyMoreUndo());
+    emit signalModified(d->undoMan->anyMoreUndo(), true);
+}
+
+void ImlibInterface::redo()
+{
+    if (!d->undoMan->anyMoreRedo())
+    {
+        emit signalModified(d->undoMan->anyMoreUndo(), false);
+        return;
+    }
+
+    d->undoMan->redo();
+    emit signalModified(true, d->undoMan->anyMoreRedo());
 }
 
 void ImlibInterface::restore()
 {
+    bool isReadOnly;
     d->undoMan->clear();
     
-    load(d->filename);
-    emit signalModified(false);
+    load(d->filename, &isReadOnly);
+    emit signalModified(false, false);
 }
 
 bool ImlibInterface::save(const QString& file, int JPEGcompression, 
@@ -373,7 +576,7 @@ bool ImlibInterface::save(const QString& file, int JPEGcompression,
     if (result)
     {
         d->undoMan->clear();
-        emit signalModified(false);
+        emit signalModified(false, false);
     }
     
     return result;
@@ -394,23 +597,30 @@ bool ImlibInterface::saveAs(const QString& file, int JPEGcompression,
     imlib_modify_color_modifier_contrast(d->contrast);
     imlib_modify_color_modifier_gamma(d->gamma);
     imlib_apply_color_modifier();
-    
+
     if (mimeType.isEmpty())
         result = saveAction(file, JPEGcompression, PNGcompression, 
                             TIFFcompression, imlib_image_format());
-    else 
-       result = saveAction(file, JPEGcompression, PNGcompression, 
+    else
+        result = saveAction(file, JPEGcompression, PNGcompression, 
                            TIFFcompression, mimeType);
-    
+
     imlib_context_pop();
 
-    if (result)
+    return result;
+}
+
+void ImlibInterface::setModified(bool val)
+{
+    if (val)
+    {
+        emit signalModified(true, true);
+    }
+    else 
     {
         d->undoMan->clear();
-        emit signalModified(false);
+        emit signalModified(false, false);
     }
-
-    return result;    
 }
 
 int ImlibInterface::width()
@@ -576,7 +786,7 @@ void ImlibInterface::rotate90(bool saveUndo)
     d->origHeight = imlib_image_get_height();    
     imlib_context_pop();
 
-    emit signalModified(true);
+    emit signalModified(true, d->undoMan->anyMoreRedo());
 }
 
 void ImlibInterface::rotate180(bool saveUndo)
@@ -595,7 +805,7 @@ void ImlibInterface::rotate180(bool saveUndo)
     d->origHeight = imlib_image_get_height();
     imlib_context_pop();
 
-    emit signalModified(true);
+    emit signalModified(true, d->undoMan->anyMoreRedo());
 }
 
 void ImlibInterface::rotate270(bool saveUndo)
@@ -614,7 +824,7 @@ void ImlibInterface::rotate270(bool saveUndo)
     d->origHeight = imlib_image_get_height();
     imlib_context_pop();
 
-    emit signalModified(true);
+    emit signalModified(true, d->undoMan->anyMoreRedo());
 }
 
 void ImlibInterface::flipHoriz(bool saveUndo)
@@ -633,7 +843,7 @@ void ImlibInterface::flipHoriz(bool saveUndo)
     d->origHeight = imlib_image_get_height();
     imlib_context_pop();
     
-    emit signalModified(true);
+    emit signalModified(true, d->undoMan->anyMoreRedo());
 }
 
 void ImlibInterface::flipVert(bool saveUndo)
@@ -652,12 +862,12 @@ void ImlibInterface::flipVert(bool saveUndo)
     d->origHeight = imlib_image_get_height();
     imlib_context_pop();
 
-    emit signalModified(true);
+    emit signalModified(true, d->undoMan->anyMoreRedo());
 }
 
 void ImlibInterface::crop(int x, int y, int w, int h)
 {
-    d->undoMan->addAction(new UndoActionIrreversible(this));
+    d->undoMan->addAction(new UndoActionIrreversible(this, "Crop"));
 
     imlib_context_push(d->context);
     imlib_context_set_image(d->image);
@@ -673,12 +883,12 @@ void ImlibInterface::crop(int x, int y, int w, int h)
     d->origHeight = imlib_image_get_height();
     imlib_context_pop();
 
-    emit signalModified(true);
+    emit signalModified(true, d->undoMan->anyMoreRedo());
 }
 
 void ImlibInterface::resize(int w, int h)
 {
-    d->undoMan->addAction(new UndoActionIrreversible(this));
+    d->undoMan->addAction(new UndoActionIrreversible(this, "Resize"));
 
     imlib_context_push(d->context);
     imlib_context_set_image(d->image);
@@ -697,12 +907,13 @@ void ImlibInterface::resize(int w, int h)
     d->origHeight = imlib_image_get_height();
     imlib_context_pop();
 
-    emit signalModified(true);
+    emit signalModified(true, d->undoMan->anyMoreRedo());
 }
 
 void ImlibInterface::changeGamma(double gamma)
 {
     d->undoMan->addAction(new UndoActionBCG(this, d->gamma, d->brightness,
+                                            d->contrast, gamma, d->brightness,
                                             d->contrast));
 
     imlib_context_push(d->context);
@@ -719,12 +930,13 @@ void ImlibInterface::changeGamma(double gamma)
     
     imlib_context_pop();
 
-    emit signalModified(true);
+    emit signalModified(true, d->undoMan->anyMoreRedo());
 }
 
 void ImlibInterface::changeBrightness(double brightness)
 {
     d->undoMan->addAction(new UndoActionBCG(this, d->gamma, d->brightness,
+                                            d->contrast, d->gamma, brightness,
                                             d->contrast));
 
     imlib_context_push(d->context);
@@ -741,13 +953,14 @@ void ImlibInterface::changeBrightness(double brightness)
 
     imlib_context_pop();
 
-    emit signalModified(true);
+    emit signalModified(true, d->undoMan->anyMoreRedo());
 }
 
 void ImlibInterface::changeContrast(double contrast)
 {
     d->undoMan->addAction(new UndoActionBCG(this, d->gamma, d->brightness,
-                                            d->contrast));
+                                            d->contrast, d->gamma, d->brightness,
+                                            contrast));
 
     imlib_context_push(d->context);
     imlib_context_set_color_modifier(d->cmod);
@@ -763,7 +976,7 @@ void ImlibInterface::changeContrast(double contrast)
 
     imlib_context_pop();
 
-    emit signalModified(true);
+    emit signalModified(true, d->undoMan->anyMoreRedo());
 }
 
 void ImlibInterface::changeBCG(double gamma, double brightness, double contrast)
@@ -784,13 +997,13 @@ void ImlibInterface::changeBCG(double gamma, double brightness, double contrast)
 
     imlib_context_pop();
 
-    emit signalModified(true);
+    emit signalModified(true, d->undoMan->anyMoreRedo());
 }
 
 
 void ImlibInterface::setBCG(double brightness, double contrast, double gamma)
 {
-    d->undoMan->addAction(new UndoActionIrreversible(this));
+    d->undoMan->addAction(new UndoActionIrreversible(this, "Brithness, Contrast, Gamma"));
 
     imlib_context_push(d->context);
 
@@ -819,7 +1032,7 @@ void ImlibInterface::setBCG(double brightness, double contrast, double gamma)
 
     imlib_context_pop();
 
-    emit signalModified(true);
+    emit signalModified(true, d->undoMan->anyMoreRedo());
 }
 
 uint* ImlibInterface::getData()
@@ -835,14 +1048,16 @@ uint* ImlibInterface::getData()
         return 0;
 }
 
-void ImlibInterface::putData(uint* data, int w, int h,
-                             bool saveUndo)
+
+void ImlibInterface::putData(const QString &caller, uint* data, int w, int h)
+{
+    d->undoMan->addAction(new UndoActionIrreversible(this, caller));
+    putData(data, w, h);
+}
+
+
+void ImlibInterface::putData(uint* data, int w, int h)
 {
-    if (saveUndo)
-    {
-        d->undoMan->addAction(new UndoActionIrreversible(this));
-    }
-    
     imlib_context_push(d->context);
     imlib_context_set_image(d->image);
 
@@ -877,7 +1092,7 @@ void ImlibInterface::putData(uint* data, int w, int h,
     
     imlib_context_pop();
 
-    emit signalModified(true);
+    emit signalModified(true, d->undoMan->anyMoreRedo());
 }
 
 uint* ImlibInterface::getSelectedData()
@@ -937,7 +1152,7 @@ void ImlibInterface::putSelectedData(uint* data, bool saveUndo)
 
     imlib_context_pop();
 
-    emit signalModified(true);
+    emit signalModified(true, d->undoMan->anyMoreRedo());
 }
 
 bool ImlibInterface::saveAction(const QString& saveFile, int JPEGcompression,
@@ -982,6 +1197,15 @@ bool ImlibInterface::saveAction(const QString& saveFile, int JPEGcompression,
         return false;  // Do not reload the file if saving failed !
     }
 
+    // file saved. now preserve the permissions
+    if (d->filePermissions != 0)
+    {
+        // Add the user write permission.
+        // There is a problem with lost Exif info on read only files,
+        // and there won't be sophisticated handling for this in the 0.8 branch any more.
+        ::chmod(QFile::encodeName(saveFile), d->filePermissions | S_IWRITE);
+    }
+
     return true;
 }
 
@@ -1093,6 +1317,16 @@ bool ImlibInterface::saveTIFF(const QString& saveFile, bool compress)
     return false;
 }
 
+void ImlibInterface::getUndoHistory(QStringList &titles)
+{
+    d->undoMan->getUndoHistory(titles);
+}
+
+void ImlibInterface::getRedoHistory(QStringList &titles)
+{
+    d->undoMan->getRedoHistory(titles);
+}
+
 ImlibInterface* ImlibInterface::instance()
 {
     if (!m_instance) {
